

#include <math.h>

#include "stdafx.h"
#include "Label.h"
#include "MyImage.h"
#include "gloabalFunctions.h"


/**The constructor for gap label
*/
Label::Label(void)
{
	ID = LABEL_GAP;
	width = 0;
	height = 0;
	image_width = 0;
	image_height = 0;
	tile_num = 0;
	locations =	NULL;
	tile_map = NULL;
}

int Label::getHeight()
{
	return height;
}

int Label::getWidth()
{
	return width;
}

int Label::getID()
{
	return ID;
}

int Label::getTileNumber()
{
	return tile_num;
}

void Label::setLabel(int I_ID, int I_width, int I_height, int I_image_width, int I_image_height, float **ori_map,float **I_locations)
{
	ID = I_ID;
	width = I_width;
	height = I_height;
  	image_width = I_image_width;
	image_height = I_image_height;
	tile_num = 0;

	if(I_locations != NULL && ori_map != NULL)
	{
		for(int i = 0; i < image_height; i++)
			for(int j = 0; j < image_width; j++)
			{
				if(I_locations[i][j] == CENTER)tile_num++;
			}
	}

	if(tile_num > 0)
	{
		tile_map = make_2D_int_array(image_height,image_width);
		for(int i = 0; i < image_height;i++)
			for(int j = 0; j < image_width;j++)tile_map[i][j] = GAP_REGION;

		locations = (MNode *)malloc(sizeof(MNode)*tile_num);
		int index = 0;
		for(int i = 0; i < image_height; i++)
			for(int j = 0; j < image_width; j++)
			{
				if(I_locations[i][j] == CENTER)
				{
					//locations[index].ID = ID;
					//locations[index].label = LABEL_GAP;
					locations[index].site.x = j;
					locations[index].site.y = i;
					float ori = ori_map[i][j];
					CPoint center, icenter;
					center.x = j;
					center.y = i;
					icenter = coordinate_change(center,0,0,ori);

					//vertex of the tile
					CPoint vertex[4];
					CPoint ivertex[4];
					vertex[0].x = icenter.x-(int)(width/2);
					vertex[0].y = icenter.y-(int)(height/2);
					vertex[1].x = icenter.x+(int)(width/2);
					vertex[1].y = icenter.y-(int)(height/2);
					vertex[2].x = icenter.x-(int)(width/2);
					vertex[2].y = icenter.y+(int)(height/2);
					vertex[3].x = icenter.x+(int)(width/2);
					vertex[3].y = icenter.y+(int)(height/2);

					int minx,miny,maxx,maxy;
					minx = miny = 1000000;
					maxx = maxy = -1;
					for(int k = 0; k < 4; k++)
					{
						ivertex[k] = inverse_coordinate_change(vertex[k],0,0,ori);
						if(ivertex[k].x < minx)minx = ivertex[k].x;
						if(ivertex[k].x > maxx)maxx = ivertex[k].x;
						if(ivertex[k].y < miny)miny = ivertex[k].y;
						if(ivertex[k].y > maxy)maxy = ivertex[k].y;
					}
					
					CPoint temp,itemp;
					for(temp.x = minx; temp.x <= maxx; temp.x++)
						for(temp.y = miny;temp.y <= maxy;temp.y++)
						{
							itemp = temp;
							if(check_boundary(&itemp,image_width,image_height))
							{
								if(inside(temp,vertex,0,0,ori)==OUTSIDE)continue;
								tile_map[itemp.y][itemp.x] = index;
							}
						}

					index++;
				}
			}
	}
}

MNode *Label::returnTiles()
{
	if(locations == NULL || tile_num == 0)return NULL;
	return locations;
}

int Label::getTileIndex(CPoint p)
{
	if(tile_map == NULL)return GAP_REGION;
	if(check_boundary(&p,image_width,image_height))
	{
		return tile_map[p.y][p.x];
	}
	else return GAP_REGION;
}

CPoint Label::returnCenter(CPoint p)
{
	int index = getTileIndex(p);
	if( index == GAP_REGION || index >= tile_num)
	{
		CPoint center;
		center.x = -1;
		center.y = -1;
		return center;
	}
	return locations[index].site;
}