#include "stdafx.h"
#include ".\mosaic.h"
//#include "integral_image.h"
#include "gloabalFunctions.h"
#include "GCoptimization.h"
#include "Node.h"
#include ".\priority_queue.h"

#include <stdlib.h>
#include <time.h>
#include <string.h>

#define THRESH 100
#define MAX_ITERATION ori_number*10

#define INTERSECT 1
#define SEPERATE 0
#define LAY_TILE 1
#define NO_TILE 0
#define LAYER 2

#define COVERED 5

Mosaic::Mosaic(MyImage *I_image,int m_width,int m_height, int ori_num)
{
	if(I_image == NULL)
	{
		printf("No Image!\n");
		exit(1);
	}
	image = I_image;

	//size of the tile
	if(m_width < 10)m_width = 10;
	if(m_height < 10)m_height = 10;
	width = m_width;
	height = m_height;
	if(ori_num < 16)ori_num = 16;
	ori_number = ori_num;


	//+
	WD = 100;
	//-
	WE = -2;
	//+
	WV = 2;
	//+
	WS = 5;

	WG = 0.0015;

	ori_map = NULL;
	ori_data = NULL;
	tileNumber = 0;
	tile_map = NULL;
	
	inter_width = (int)(sqrt((float)(width*width+height*height))+width+0.5)+1;
	inter_height = (int)(sqrt((float)(width*width+height*height))+height+0.5)+1;
	CENTER_X = (int)(sqrt((float)(width*width+height*height))/2+(float)width/2 +0.5);
	CENTER_Y = (int)(sqrt((float)(width*width+height*height))/2+(float)height/2 +0.5);;
	compute_intersect();

	//compute the gradient
	gradient_map = compute_gradient(image,&edge_map);
	for(int j = 0; j < image->m_width; j++)
	{
		gradient_map[height][j] = 100;
		gradient_map[image->m_height-height][j]=100;
	}
	for(int i = 0; i < image->m_height; i++)
	{
		gradient_map[i][width] = 100;
		gradient_map[i][image->m_width-width] = 100;
	}
	labelSet = NULL;
}

Mosaic::~Mosaic(void)
{
}

void Mosaic::generateOri()
{
	clock_t start_time, finish_time;
	start_time = clock();
	
	if(gradient_map == NULL || edge_map == NULL || width < 10 || height < 10)return;

	printf("Begin building orientation map...\n");

	int image_width = image->m_width;
	int image_height = image->m_height;

	if( ori_map == NULL )ori_map = make_2D_float_array(image_height,image_width);
	if( ori_data == NULL )ori_data = make_2D_float_array(image_height,image_width);
	for(int i = 0; i < image_height;i++)
		for(int j = 0; j < image_width; j++)
		{
			ori_map[i][j] = 0;
			ori_data[i][j] = 0;
		}

	//compute integral images
	for(int i = 0; i < ori_number; i++)
	{
		iImages[i].computeIntegralImage(gradient_map,image_width,image_height,float(i*PI/(2*ori_number)));
	}
	printf("Finish building integral images\n");

		//Compute distance map
	for(int i = 0; i < image_height;i++)
		for(int j = 0; j < image_width; j++)
		{
			if(gradient_map[i][j] < 5)gradient_map[i][j] = 1000000;
		}

	float **distance_map = computeDistanceMap(gradient_map,image_width,image_height);

	//Compute the gradient of distance map,edge_map now is the edge map by distance transform!
	float graX,graY;
	for(int i = 1; i < image_height-1; i++)
		for(int j = 1; j < image_width-1; j++)
		{
			graX = (distance_map[i-1][j-1]-distance_map[i-1][j+1])+2*(distance_map[i][j-1]-distance_map[i][j+1])+(distance_map[i+1][j-1]-distance_map[i+1][j+1]);
			graY = (distance_map[i-1][j-1]-distance_map[i+1][j-1])+2*(distance_map[i-1][j]-distance_map[i+1][j])+(distance_map[i-1][j+1]-distance_map[i+1][j+1]);
			edge_map[i][j] = atan2(graY,graX);
			if(edge_map[i][j] < 0-PI/2)edge_map[i][j]+=PI;
			else if(edge_map[i][j] < 0)edge_map[i][j]+=PI/2;
			else if (edge_map[i][j] > PI/2)edge_map[i][j]-=PI/2;
		}
		//Now the edge_map is the angles of distance map
	printf("Finish building distance map...\n");

	GCoptimization *graph =  new GCoptimization(image_width,image_height,ori_number,SET_ALL_AT_ONCE,SET_ALL_AT_ONCE);

	int *data_term = (int *)malloc(sizeof(int)*image_width*image_height*ori_number);
	int *smooth_term = (int *)malloc(sizeof(int)*ori_number*ori_number);
	float step = PI/(float)(2*ori_number);
	
	//Set the data term
	CPoint temp;
	for(temp.y = 0; temp.y < image_height; temp.y++)
		for(temp.x = 0; temp.x < image_width; temp.x++)
		{
			for(int i = 0; i < ori_number; i++)
			{
				int index = ((temp.y*image_width)+temp.x)*ori_number+i;
				data_term[index] = 0;
				float up,left,right,below;
				up = left = right = below = 0;
				float ori = i*step;
				float variance = 0;
				if(if_complete(temp,i*step,image_width,image_height,width,height))
				{
					CPoint temp2;
					CPoint temp3;
					CPoint icenter = coordinate_change(temp,0,0,ori);
					//Process the upper boundary
					temp2.x = icenter.x;
					temp2.y = icenter.y - (int)(3*height/4);
					up = iImages[i].getValue(temp2,width,(int)(height/2));

					//below
					temp2.x = icenter.x;
					temp2.y = icenter.y+(int)(3*height/4);
					below = iImages[i].getValue(temp2,width,(int)(height/2));

					//left
					temp2.x = icenter.x -(int)(3*width/4);
					temp2.y = icenter.y;
					left = iImages[i].getValue(temp2,(int)(width/2),height);

					//right
					temp2.x = icenter.x +(int)(3*width/4);
					temp2.y = icenter.y;
					right = iImages[i].getValue(temp2,(int)(width/2),height);

					data_term[index] = data_term[index]+(up+below+right+left)*(WE);

					//variance
					variance = iImages[i].getValue(icenter,width,height);
					data_term[index] = data_term[index]+WV*variance;
					float ori_dif;
					if(edge_map[temp.y][temp.x] > 0)ori_dif = fabs(ori-edge_map[temp.y][temp.x]);
					else ori_dif = fabs(ori-(edge_map[temp.y][temp.x]+PI));
					data_term[index] = data_term[index] + WD*ori_dif;
				}
				//Give the bias towards the horizontal and vertical orientation at the border of the image
				if(temp.y <= (int)(height) ||(image_height - temp.y <= (int)(height))|| temp.x <= (int)(width)|| (image_width-temp.x) <= (int)(width))
				{
					if(ori == 0)data_term[index] -= 200;
					//else data_term[index]= 1000;
				}
			}
			GCoptimization::PixelType pixel = temp.y*image_width+temp.x;
			graph->setLabel(pixel,0);
		}

	for(int i = 0; i < ori_number; i++)
		for(int j = 0; j < ori_number; j++)
		{
			int index = ori_number*i+j;
			if(abs(i-j)<(int)(ori_number/2))
				smooth_term[index] = abs(i-j)*WS;
				//smooth_term[index] = WS*(1-pow(SC,0-(i-j)*(i-j)*DELTA));
				//smooth_term[index] = WS*pow(abs(i-j),5);
			else smooth_term[index]=(ori_number-abs(i-j))*WS;
				//smooth_term[index] = WS*(1-pow(SC,0-(ori_number-abs(i-j))*(ori_number-abs(i-j))*DELTA));
				//smooth_term[index] = WS*pow((ori_number-abs(i-j)),5);
		}

		graph->setData(data_term);
		graph->setSmoothness(smooth_term);
		printf("Finish building graph!\n");

		graph->expansion();
		printf("Finish optimization!\n");

	for(int i = 0; i < image_height; i++)
		for(int j = 0; j < image_width;j++)
		{
			GCoptimization::PixelType pixel =  i*image_width+j;
			ori_map[i][j] = (float)(graph->whatLabel(pixel)*step);
			int index = ((i*image_width)+j)*ori_number+(int)graph->whatLabel(pixel);
			ori_data[i][j] = data_term[index];
		}
	
	finish_time = clock();
	double duration = (double)(finish_time-start_time)/CLOCKS_PER_SEC;

	printf("Time of generating orientations:\t %lf seconds \n",duration);
	//visualizeData();
	free_2D_float_array(distance_map);
	free(data_term);
	free(smooth_term);
	delete graph;
}

void Mosaic::visualizeOri()
{
	if(ori_map == NULL)
	{
		printf("No orientation map!\n");
		return;
	}
	int image_width = image->m_width;
	int image_height = image->m_height;

	MyImage *im = new MyImage(image_height,image_width,COLOR,BMP_TYPE,FLOAT_FORMAT);
	for(int i = 0; i < image_height;i++)
		for(int j = 0; j < image_width; j++)
		{
			CPoint temp;
			temp.x = j;
			temp.y = i;
			imRef_R(im,temp) = imRef_R(image,temp);
			imRef_G(im,temp) = imRef_G(image,temp);
			imRef_B(im,temp) = imRef_B(image,temp);
		}

	for(int i = 0; i < image_height-height; i+=height)
	{
		for(int j = 0; j < image_width-width; j+=width)
		{
			CPoint center;
			center.x = (int)(j+width/2);
			center.y = (int)(i+height/2);
			float ori = ori_map[center.y][center.x];
			//Vertical orientation
			if(ori >= 3.1415926/2 && ori <= 3.1415927/2)
			{
				CPoint temp;
				temp.x = center.x;
				for(temp.y = i; temp.y < i + height; temp.y++)
				{
					imRef_R(im,temp) = 196;
					imRef_G(im,temp) = 0;
					imRef_B(im,temp) = 91;
				}
				continue;
			}
			//Horizontal orientation
			if(ori <= 0.0000001 || ori >= 3.1415926)
			{
				CPoint temp;
				temp.y = center.y;
				for(temp.x = j; temp.x < j+width;temp.x++)
				{
					imRef_R(im,temp) = 196;
					imRef_G(im,temp) = 0;
					imRef_B(im,temp) = 91;
				}
				continue;
			}

			float a,b;
			a = tan(ori);
			b = center.y - a*center.x;
			CPoint point[4];
			point[0].y = i+2;
			point[0].x = (int)((point[0].y-b)/a+0.5);
			point[1].x = j+width-3;
			point[1].y = (int)((point[1].x*a+b)+0.5);
			point[2].y = i+height-3;
			point[2].x = (int)((point[2].y-b)/a+0.5);
			point[3].x = j+2;
			point[3].y = (int)((point[3].x*a+b)+0.5);

			CPoint termi[2];
			int find = 0;
			for(int k = 0; k < 4; k++)
			{
				if(point[k].x < j+width && point[k].x >= j && point[k].y <i+height && point[k].y >= i)
				{
					if(find >1)break;
					termi[find++] = point[k];
				}
			}

			if(find < 2)continue;
			float disY,disX;
			disY = float(termi[1].y - termi[0].y)/100;
			disX = float(termi[1].x - termi[0].x)/100;
			CPoint last,now;
			last = termi[0];
			for(int k = 0; k <= 100; k++)
			{
				now.x = (int)((float)(termi[0].x)+k*disX);
				now.y = (int)((float)(termi[0].y)+k*disY);
				if(now != last)
				{
					last = now;
					imRef_R(im,now) = 196;
					imRef_G(im,now) = 0;
					imRef_B(im,now) = 91;
				}
			}
		}
	}
	im->WriteImage("ori_map.bmp");
}

void Mosaic::generateMosaic()
{
	//generateOri();
	layTile();
}

void Mosaic::visualizeData()
{
	if(ori_map == NULL || ori_data == NULL)
	{
		printf("No data term avaliabel!\n");
		return;
	}

	int col = image->m_width;
	int row = image->m_height;
	MyImage *im = new MyImage(row,col,GRAY,BMP_TYPE,BYTE_FORMAT);
	CPoint temp;

	float min = 10000000;
	float max = -10000000;

	for( int i = 0; i < row; i++)
		for(int j = 0; j < col; j++)
		{
			if(ori_data[i][j] < min)min = ori_data[i][j];
			if(ori_data[i][j] > max)max = ori_data[i][j];
		}

	float dis = fabs(max-min);

	for(int i = 0; i < row; i++)
		for(int j = 0; j < col; j++)
		{
			temp.x = j;
			temp.y = i;
			im->m_data[i][j] = (fabs(ori_data[i][j]-min)/dis)*255*1.5;
		}
		im->WriteImage("data_term.bmp");
}

void Mosaic::visualizeTile(char *fileName)
{
	if(tileNumber == 0 || tile_map == NULL)
	{
		printf("No tile map!\n");
		return;
	}

	MyImage *im = new MyImage(image->m_height,image->m_width,COLOR,BMP_TYPE,FLOAT_FORMAT);
	for(int i = 0; i < image->m_height;i++)
		for(int j = 0; j < image->m_width; j++)
		{
			CPoint temp;
			temp.x = j;
			temp.y = i;
			imRef_R(im,temp) = 140;
			imRef_G(im,temp) = 140;
			imRef_B(im,temp) = 140;
		}

	CPoint center;
	for(center.y = 0; center.y < image->m_height; center.y++)
		for(center.x = 0; center.x < image->m_width;center.x++)
		{
			if(tile_map[center.y][center.x] != NO_TILE)
			{
				float R,G,B;
				R = G = B = 0;
				int pNumber = 0;
				CPoint temp,temp2;
				float ori = ori_map[center.y][center.x];
				CPoint icenter = coordinate_change(center,0,0,ori);
				if(tile_map[center.y][center.x] == LAY_TILE)
				{
					for(temp.y = icenter.y-(int)(height/2);temp.y < icenter.y+(int)(height/2);temp.y++)
						for(temp.x = icenter.x-(int)(width/2);temp.x < icenter.x+(int)(width/2);temp.x++)
						{
							temp2 = inverse_coordinate_change(temp,0,0,ori);
							if(check_boundary(&temp2,image->m_width,image->m_height))
							{
								R += imRef_R(image,temp2);
								G += imRef_G(image,temp2);
								B += imRef_B(image,temp2);
								pNumber++;
							}
						}
					R = R/(float)pNumber;
					G = G/(float)pNumber;
					B = B/(float)pNumber;
				}
				else
				{
					R = 255;
					G = 0;
					B = 255;
				}

				CPoint vertex[4];
				vertex[0].x = icenter.x-(int)(width/2);
				vertex[0].y = icenter.y-(int)(height/2);
				vertex[1].x = icenter.x+(int)(width/2);
				vertex[1].y = icenter.y-(int)(height/2);
				vertex[2].x = icenter.x-(int)(width/2);
				vertex[2].y = icenter.y+(int)(height/2);
				vertex[3].x = icenter.x+(int)(width/2);
				vertex[3].y = icenter.y+(int)(height/2);

				int minx, miny, maxx,maxy;
				minx = miny = 1000000;
				maxx = maxy = -1000000;

				for(int k = 0; k < 4; k++)
				{
					temp = inverse_coordinate_change(vertex[k],0,0,ori);
					if(temp.x < minx )minx = temp.x;
					if(temp.x > maxx )maxx = temp.x;
					if(temp.y < miny )miny = temp.y;
					if(temp.y > maxy )maxy = temp.y;
				}

				for(temp.y = miny; temp.y <= maxy; temp.y++)
					for(temp.x = minx; temp.x <= maxx; temp.x++)
					{
						temp2 = temp;
						if(check_boundary(&temp2,image->m_width,image->m_height))
						{
							int in = inside(temp,vertex,0,0,ori); 
							if( in == INSIDE)
							{
									imRef_R(im,temp2) = R;
									imRef_G(im,temp2) = G;
									imRef_B(im,temp2) = B;
							}
							if(in == BORDER)
							{
								imRef_R(im,temp2) = 140;
								imRef_G(im,temp2) = 140;
								imRef_B(im,temp2) = 140;
							}
						}
					}

			}
		}
/*		char fileName[128];
		char fileNum[4];
		_itoa(iterate,fileNum,10);
		strcpy(fileName,"result");
		strcat(fileName,fileNum);
		strcat(fileName,".bmp");*/
		if(fileName == NULL)
		{
			fileName = new char[128];
			strcpy(fileName, "mosaic.bmp");
		}

		im->WriteImage(fileName);
		/*FILE *record = fopen("tile_record.txt","w");
		fprintf(record,"width:\t %d\t height:\t %d\t tileNumber:\t %d\n",width,height,tileNumber);
		fprintf(record,"WE:\t %d\t WV:\t %d\t WD:\t %d, WS:\t %d\t WG:\t %f\n", WE,WV,WD,WS,WG);
		fclose(record);*/

}


void Mosaic::visualizeTile(int iterate)
{
	if(tileNumber == 0 || tile_map == NULL)
	{
		printf("No tile map!\n");
		return;
	}

	MyImage *im = new MyImage(image->m_height,image->m_width,COLOR,BMP_TYPE,FLOAT_FORMAT);
	for(int i = 0; i < image->m_height;i++)
		for(int j = 0; j < image->m_width; j++)
		{
			CPoint temp;
			temp.x = j;
			temp.y = i;
			imRef_R(im,temp) = 140;
			imRef_G(im,temp) = 140;
			imRef_B(im,temp) = 140;
		}

	CPoint center;
	for(center.y = 0; center.y < image->m_height; center.y++)
		for(center.x = 0; center.x < image->m_width;center.x++)
		{
			if(tile_map[center.y][center.x] != NO_TILE)
			{
				float R,G,B;
				R = G = B = 0;
				int pNumber = 0;
				CPoint temp,temp2;
				float ori = ori_map[center.y][center.x];
				CPoint icenter = coordinate_change(center,0,0,ori);
				if(tile_map[center.y][center.x] == LAY_TILE)
				{
					for(temp.y = icenter.y-(int)(height/2);temp.y < icenter.y+(int)(height/2);temp.y++)
						for(temp.x = icenter.x-(int)(width/2);temp.x < icenter.x+(int)(width/2);temp.x++)
						{
							temp2 = inverse_coordinate_change(temp,0,0,ori);
							if(check_boundary(&temp2,image->m_width,image->m_height))
							{
								R += imRef_R(image,temp2);
								G += imRef_G(image,temp2);
								B += imRef_B(image,temp2);
								pNumber++;
							}
						}
					R = R/(float)pNumber;
					G = G/(float)pNumber;
					B = B/(float)pNumber;
				}
				else
				{
					R = 255;
					G = 0;
					B = 255;
				}

				CPoint vertex[4];
				vertex[0].x = icenter.x-(int)(width/2);
				vertex[0].y = icenter.y-(int)(height/2);
				vertex[1].x = icenter.x+(int)(width/2);
				vertex[1].y = icenter.y-(int)(height/2);
				vertex[2].x = icenter.x-(int)(width/2);
				vertex[2].y = icenter.y+(int)(height/2);
				vertex[3].x = icenter.x+(int)(width/2);
				vertex[3].y = icenter.y+(int)(height/2);

				int minx, miny, maxx,maxy;
				minx = miny = 1000000;
				maxx = maxy = -1000000;

				for(int k = 0; k < 4; k++)
				{
					temp = inverse_coordinate_change(vertex[k],0,0,ori);
					if(temp.x < minx )minx = temp.x;
					if(temp.x > maxx )maxx = temp.x;
					if(temp.y < miny )miny = temp.y;
					if(temp.y > maxy )maxy = temp.y;
				}

				for(temp.y = miny; temp.y <= maxy; temp.y++)
					for(temp.x = minx; temp.x <= maxx; temp.x++)
					{
						temp2 = temp;
						if(check_boundary(&temp2,image->m_width,image->m_height))
						{
							int in = inside(temp,vertex,0,0,ori); 
							if( in == INSIDE)
							{
									imRef_R(im,temp2) = R;
									imRef_G(im,temp2) = G;
									imRef_B(im,temp2) = B;
							}
							if(in == BORDER)
							{
								imRef_R(im,temp2) = 140;
								imRef_G(im,temp2) = 140;
								imRef_B(im,temp2) = 140;
							}
						}
					}

			}
		}
		char fileName[128];
		char fileNum[4];
		_itoa(iterate,fileNum,10);
		strcpy(fileName,"result");
		strcat(fileName,fileNum);
		strcat(fileName,".bmp");

		im->WriteImage(fileName);
}

void Mosaic::changeParameter(int wv, int we, int wd, int ws, float wg)
{
	WV = wv;
	WE = we;
	WD = wd;
	WS = ws;
	WG = wg;
}

void Mosaic::layTile()
{

	clock_t start_time, finish_time;
	start_time = clock();
	if(labelSet == NULL)tile_map = buildLabelsFast();
	if(tile_map == NULL)return;
	int tile_num = 0;

	CPoint *sites;
	int shorter = (width < height?width:height);
	int maxSites = ((int)(image->m_height/shorter)+1)*((int)(image->m_width/shorter)+1);
	sites = (CPoint *)malloc(sizeof(CPoint)*maxSites);
	CPoint *newSites = (CPoint *)malloc(sizeof(CPoint)*maxSites);
	//array to store the nonoverlap region
	int *oldInt = (int *)malloc(sizeof(int)*maxSites);
	int *newInt = (int *)malloc(sizeof(int)*maxSites);
	//nodes in graph
	Graph::node_id *new_graph_nodes = (Graph::node_id *)malloc(sizeof(Graph::node_id)*maxSites);
	Graph::node_id *old_graph_nodes = (Graph::node_id *)malloc(sizeof(Graph::node_id)*maxSites);

	//Initialize label_map
	for(int i = 0; i < image->m_height; i++)
		for(int j = 0; j < image->m_width; j++)
		{
			if(tile_map[i][j] == LAY_TILE)
			{
				sites[tile_num].x = j;
				sites[tile_num].y = i;
				tile_num++;
			}
		}
	tileNumber = tile_num;

	//The array to store the usage of each layer
	int label_num = labelSet->getLabelNum();
	if(label_num == -1)label_num = MAX_LABEL;
	int *extand = (int *)malloc(sizeof(int)*label_num);
	for(int i = 0; i < label_num; i++)extand[i] = 0;

	bool finish = false;
	Graph::flowtype oldflow = 1000000000;
	int iterate = 0;
	float **g_map;
	int last_tile_num = 0;
	float step = PI/(2*ori_number);

	srand(1000);

	while(!finish)
	{
		clock_t start_time, finish_time;
		start_time = clock();
		int l = (abs(rand()))%(label_num-1);
		Label *label = labelSet->getLabel(l);
		if(label == NULL)continue;
		extand[l]++;
		MNode *labelSites = label->returnTiles();
		int siteNum = label->getTileNumber();
		if(labelSites == NULL || siteNum < 1)continue;
		printf("Label:\t%d\tNumber of sites:\t%d\n",l,siteNum);

		for(int i = 0; i < maxSites;i++)
		{
			oldInt[i] = width*height;
			newInt[i] = width*height;
		}

		//Two arrays to store site location:: RELEASE MEMORY!!!!
		int grid_width = (int)((float)(image->m_width)/(float)width+0.5)+1;
		int grid_height = (int)((float)(image->m_height)/(float)width+0.5)+1;
		Queue **oldGrid = (Queue **)malloc(sizeof(Queue *)*grid_width*grid_height);
		Queue **newGrid = (Queue **)malloc(sizeof(Queue *)*grid_width*grid_height);
		for (int i = 0; i< grid_width*grid_height; i++)
		{
			oldGrid[i] = new Queue;
			newGrid[i] = new Queue;
		}

		for(int i = 0; i < siteNum;i++)
		{
			CPoint center = labelSites[i].site;
			int grid_index = ((int)(center.y/height))*grid_width+((int)(center.x/width));
			newGrid[grid_index]->inqueue(center,i);
		}

		for(int i = 0; i < tile_num; i++)
		{
			CPoint oldCenter = sites[i];
			int grid_index = ((int)(oldCenter.y/height))*grid_width+((int)(oldCenter.x/width));
			oldGrid[grid_index]->inqueue(oldCenter,i);
		}

		//build graph and add nodes
		Graph *g = new Graph();		

		//add nodes to present the old tiles
		for(int i = 0; i < tile_num; i++)
		{
			old_graph_nodes[i] = g->add_node();
		}
		for(int i = 0; i < siteNum; i++)
		{
			new_graph_nodes[i] = g->add_node();
		}

		//Processing old tiles
		for(int i = 0; i < tile_num; i++)
		{
			//Get vairiance 
			CPoint center;
			center = sites[i];
			float ori = ori_map[center.y][center.x];
			CPoint icenter = coordinate_change(center,0,0,ori);
			int ori_index = (int)(ori/step);
			if(ori_index < 0)ori_index+=ori_number;
			if(ori_index >= ori_number)ori_index-=ori_number;
			float gradient = iImages[ori_index].getValue(icenter,width,height);
			
			int grid_X = (int)(center.x/width);
			int grid_Y = (int)(center.y/height);

			//Search the neighboring grids
			for(int y = grid_Y-2; y < grid_Y+2; y++)
				for(int x = grid_X-2; x < grid_X+2; x++)
				{
					CPoint temp;
					temp.x = x;
					temp.y = y;
					if(check_boundary(&temp,grid_width,grid_height))
					{
						int grid_index = y*grid_width+x;
						Node *tempNode;
						for(tempNode = newGrid[grid_index]->Head(); tempNode != NULL;tempNode = tempNode->next)
						{
							CPoint newCenter = tempNode->point;
							int newIndex = tempNode->weight;
							if(ifIntersect(center,newCenter))
							{
								int overlap = overlapSize(center,newCenter);
								oldInt[i]-=overlap;
								g->add_edge(old_graph_nodes[i],new_graph_nodes[newIndex],1000000000,overlap);
							}
						}
					}
				}
			if(oldInt[i] < 0)oldInt[i] = 0;
			g->set_tweights(old_graph_nodes[i],oldInt[i],gradient*WG);	
		}
		
		//Processing new tiles
		for(int i = 0; i < siteNum; i++)
		{
			CPoint center;
			center = labelSites[i].site;
			float ori = ori_map[center.y][center.x];
			CPoint icenter = coordinate_change(center,0,0,ori);
			int ori_index = (int)(ori/step);
			if(ori_index < 0)ori_index+=ori_number;
			if(ori_index >= ori_number)ori_index-=ori_number;
			float gradient = iImages[ori_index].getValue(icenter,width,height);
			
			int grid_X = (int)(center.x/width);
			int grid_Y = (int)(center.y/height);

			//Search the neighboring grids
			for(int y = grid_Y-2; y < grid_Y+2; y++)
				for(int x = grid_X-2; x < grid_X+2; x++)
				{
					CPoint temp;
					temp.x = x;
					temp.y = y;
					if(check_boundary(&temp,grid_width,grid_height))
					{
						int grid_index = y*grid_width+x;
						Node *tempNode;
						for(tempNode = oldGrid[grid_index]->Head(); tempNode != NULL;tempNode = tempNode->next)
						{
							CPoint oldCenter = tempNode->point;
							int oldIndex = tempNode->weight;
							if(ifIntersect(center,oldCenter))
							{
								int overlap = overlapSize(center,oldCenter);
								newInt[i]-=overlap;
								g->add_edge(old_graph_nodes[oldIndex],new_graph_nodes[i],1000000000,overlap);
							}
						}
					}
				}
			if(newInt[i] < 0)newInt[i] = 0;
			g->set_tweights(new_graph_nodes[i],gradient*WG,newInt[i]);
		}


		Graph::flowtype newEnergy = g->maxflow();
		iterate++;

		int nSiteNum = 0;

		int change = 0;
		for(int i = 0; i < siteNum; i++)
		{
			if(g->what_segment(new_graph_nodes[i])==Graph::SINK)
			{
				change++;
				CPoint center = labelSites[i].site;
				newSites[nSiteNum] = center;
				nSiteNum++;
			}
		}

		int oldIndex = nSiteNum;
		for(int i = 0; i < tile_num; i++)
		{
			if(g->what_segment(old_graph_nodes[i]) == Graph::SINK)continue;
			change++;
			CPoint center = sites[i];
			newSites[nSiteNum] = center;
			nSiteNum++;
		}

		if((nSiteNum - tile_num < -3 && iterate > 10)||iterate >= 100)break;
		
		//To do
		tile_num = nSiteNum;
		tileNumber = nSiteNum;
		if(tile_num-last_tile_num > 9)
		{
			last_tile_num = tile_num;
		}
		for(int i = 0; i < nSiteNum; i++)
			sites[i] = newSites[i];

		finish_time = clock();
		double duration = (double)(finish_time-start_time)/CLOCKS_PER_SEC;
		printf("Time:\t %lf seconds \n",duration);
		printf("New Tile Number:\t%d\n\n",tile_num);

		for (int i = 0; i< grid_width*grid_height; i++)
		{
			for(Node *tempNode = oldGrid[i]->Head();tempNode != NULL;)
			{
				Node *next = tempNode->next;
				delete tempNode;
				tempNode = next;
			}
			for(Node *tempNode = newGrid[i]->Head();tempNode != NULL;)
			{
				Node *next = tempNode->next;
				delete tempNode;
				tempNode = next;
			}
			delete (oldGrid[i]);
			delete (newGrid[i]);
		}
	}

	for(int i = 0; i < image->m_height; i ++)
		for(int j = 0; j < image->m_width; j++)
		{
			tile_map[i][j] = NO_TILE;
		}
	
	tileNumber = tile_num;
	for(int i = 0; i < tileNumber; i++)
	{
		tile_map[sites[i].y][sites[i].x] = LAY_TILE;
	}
	free(sites);
	free(old_graph_nodes);

}

void Mosaic::compute_gap()
{
}

void Mosaic::compute_intersect()
{
	//get the size of the region to be computed
	int i_width = (int)(sqrt((float)(width*width+height*height))+width+0.5)+1;
	int i_height = (int)(sqrt((float)(width*width+height*height))+height+0.5)+1;


	intersect = (int *)malloc(sizeof(int)*i_width*i_height*ori_number);

	float step = PI/(2*(float)ori_number); 

	CPoint centerVertex[4];
	//    |0  1|
	//    |2  3| 
	//
	centerVertex[0].x = (int)(sqrt((float)(width*width+height*height))/2+0.5);
	centerVertex[0].y = (int)(sqrt((float)(width*width+height*height))/2+0.5);
	centerVertex[1].x = (int)(sqrt((float)(width*width+height*height))/2+0.5)+width-1;
	centerVertex[1].y = (int)(sqrt((float)(width*width+height*height))/2+0.5);
	centerVertex[2].x = (int)(sqrt((float)(width*width+height*height))/2+0.5);
	centerVertex[2].y = (int)(sqrt((float)(width*width+height*height))/2+0.5)+height-1;
	centerVertex[3].x = (int)(sqrt((float)(width*width+height*height))/2+0.5)+width-1;
	centerVertex[3].y = (int)(sqrt((float)(width*width+height*height))/2+0.5)+height-1;

	CPoint ivertex[4];
	ivertex[0].x = -(int)(width/2);
	ivertex[0].y = -(int)(height/2);
	ivertex[1].x = (int)(width/2);
	ivertex[1].y = -(int)(height/2);
	ivertex[2].x = -(int)(width/2);
	ivertex[2].y = (int)(height/2);
	ivertex[3].x = (int)(width/2);
	ivertex[3].y = (int)(height/2);

	for(int i = 0; i < i_height; i++)
		for(int j = 0; j < i_width; j++)
			for(int o = 0; o < ori_number; o++)
			{
				int index = (i*i_width+j)*ori_number+o;
				intersect[index] = 0;
            
				for(int y = centerVertex[0].y; y <= centerVertex[3].y; y++)
					for(int x = centerVertex[0].x; x <= centerVertex[3].x; x++)
					{
						CPoint temp;
						temp.x = (int)((x-j)*cosf(o*step)+(y-i)*sinf(o*step)+0.5);
						temp.y = (int)((y-i)*cosf(o*step)-(x-j)*sinf(o*step)+0.5);
						if(temp.x <= ivertex[3].x && temp.x >= ivertex[0].x && temp.y <= ivertex[3].y && temp.y >= ivertex[0].y)
						{
							intersect[index]++;
						}
					}
			}
}

int Mosaic::overlapSize(CPoint center1, CPoint center2)
{
	if(intersect == NULL || ori_map == NULL)
	{
		printf("No orienatation map!\n");
		return 0;
	}

	double ori1, ori2;
	if(check_boundary(&center1,image->m_width,image->m_height))ori1 = (double)ori_map[center1.y][center1.x];
	else return 0;
	if(check_boundary(&center2,image->m_width,image->m_height))ori2 = (double)ori_map[center2.y][center2.x];
	else return 0;

	center2 = center2-center1;
	center2.x += CENTER_X;
	center2.y += CENTER_Y;
	if(!check_boundary(&center2,inter_width,inter_height))
	{
		return 0;
	}

	float step = PI/(float)(2*ori_number);
	int ori_index1 = (int)(ori1/step);
	if(ori_index1 >= ori_number)ori_index1 -= ori_number;
	if(ori_index1 < 0) ori_index1 += ori_number;
	
	int ori_index2 = (int)(ori2/step);
	if(ori_index2 >= ori_number)ori_index2 -= ori_number;
	if(ori_index2 < 0) ori_index2 += ori_number;

	int ori = ori_index2-ori_index1;
	if(ori < 0)ori += ori_number;
	if(ori >= ori_number) ori -= ori_number;

	int index = (center2.y*inter_width+center2.x)*ori_number+ori;
	return intersect[index];
}

//return false if the two tiles located at center1 and center2 are sperate, otherwise return true
bool Mosaic::ifIntersect(CPoint center1,CPoint center2)
{
	if(intersect == NULL || ori_map == NULL)
	{
		printf("No orienatation map!\n");
		return true;
	}

	double ori1, ori2;
	if(check_boundary(&center1,image->m_width,image->m_height))ori1 = (double)ori_map[center1.y][center1.x];
	else return false;
	if(check_boundary(&center2,image->m_width,image->m_height))ori2 = (double)ori_map[center2.y][center2.x];
	else return false;

	center2 = center2-center1;
	center2.x += CENTER_X;
	center2.y += CENTER_Y;
	if(!check_boundary(&center2,inter_width,inter_height))
	{
		return false;
	}

	float step = PI/(float)(2*ori_number);
	int ori_index1 = (int)(ori1/step);
	if(ori_index1 >= ori_number)ori_index1 -= ori_number;
	if(ori_index1 < 0) ori_index1 += ori_number;
	
	int ori_index2 = (int)(ori2/step);
	if(ori_index2 >= ori_number)ori_index2 -= ori_number;
	if(ori_index2 < 0) ori_index2 += ori_number;

	int ori = ori_index2-ori_index1;
	if(ori < 0)ori += ori_number;
	if(ori >= ori_number) ori -= ori_number;

	int index = (center2.y*inter_width+center2.x)*ori_number+ori;
	if(intersect[index] > (float)(width*height)/15)return true;
	return false;
}

float Mosaic::gapSize(CPoint center1,CPoint center2)
{
	return 0;
}

void Mosaic::visualizeLabel(int label)
{
	if(labelSet == NULL)buildLabels();

	int labelNum = labelSet->getLabelNum();
	if(labelNum == 0)
	{
		printf("No Label\n");
		return;
	}
	if(labelNum == -1)labelNum = MAX_LABEL;
	if(label < 0 || label >= labelNum)
	{
		printf("No such label\n");
	}
	Label *layer = labelSet->getLabel(label);
	int tNumber = layer->getTileNumber();
	if(tNumber < 1)return;
	MNode *tiles = layer->returnTiles();
	if(tiles == NULL)return;

	int image_width = image->m_width;
	int image_height = image->m_height;
	MyImage *im = new MyImage(image_height,image_width,COLOR,BMP_TYPE,BYTE_FORMAT);
	CPoint temp;
	for(int i = 0; i < image_height; i++)
		for(int j = 0; j < image_width; j++)
		{	
			temp.x = j;
			temp.y = i;
			imRef_R(im,temp) = 140;
			imRef_G(im,temp) = 140;
			imRef_B(im,temp) = 140;
		}

	for(int i = 0; i < tNumber; i++)
	{
		CPoint center = tiles[i].site;
		float ori = ori_map[center.y][center.x];
		CPoint icenter = coordinate_change(center,0,0,ori);
		float R,G,B;
		R = 0;
		G = 0;
		B = 0;
		CPoint temp2;
		int pNum = 0;
		for(temp.y = icenter.y-(int)(height/2);temp.y < icenter.y+(int)(height/2);temp.y++)
			for(temp.x = icenter.x-(int)(width/2);temp.x < icenter.x+(int)(width/2);temp.x++)
			{
				temp2 = inverse_coordinate_change(temp,0,0,ori);
				if(check_boundary(&temp2,image_width,image_height))
				{
					R += imRef_R(image,temp2);
					G += imRef_G(image,temp2);
					B += imRef_B(image,temp2);
					pNum++;
				}
			}
		R = R/pNum;
		G = G/pNum;
		B = B/pNum;
		
		//vertex of the tile
		CPoint vertex[4];
		CPoint ivertex[4];
		vertex[0].x = icenter.x-(int)(width/2);
		vertex[0].y = icenter.y-(int)(height/2);
		vertex[1].x = icenter.x+(int)(width/2);
		vertex[1].y = icenter.y-(int)(height/2);
		vertex[2].x = icenter.x-(int)(width/2);
		vertex[2].y = icenter.y+(int)(height/2);
		vertex[3].x = icenter.x+(int)(width/2);
		vertex[3].y = icenter.y+(int)(height/2);

		int minx,miny,maxx,maxy;
		minx = miny = 1000000;
		maxx = maxy = -1;
		for(int k = 0; k < 4; k++)
		{
			ivertex[k] = inverse_coordinate_change(vertex[k],0,0,ori);
			if(ivertex[k].x < minx)minx = ivertex[k].x;
			if(ivertex[k].x > maxx)maxx = ivertex[k].x;
			if(ivertex[k].y < miny)miny = ivertex[k].y;
			if(ivertex[k].y > maxy)maxy = ivertex[k].y;
		}
					
		CPoint itemp;
		for(temp.x = minx; temp.x <= maxx; temp.x++)
			for(temp.y = miny;temp.y <= maxy;temp.y++)
			{
				itemp = temp;
				if(check_boundary(&itemp,image_width,image_height))
				{
					if(inside(temp,vertex,0,0,ori)==INSIDE)
					{
						imRef_R(im,temp) = R;
						imRef_G(im,temp) = G;
						imRef_B(im,temp) = B;
					}
				}
			}
	}

	char fileName[128];
	strcpy(fileName,"label");
	char fileNumber[3];
	_itoa(label,fileNumber,10);
	strcat(fileName,fileNumber);
	strcat(fileName,".bmp");
	im->WriteImage(fileName);

}

int **Mosaic::buildLabelsFast()
{
	if(image == NULL)
	{
		return NULL;
	}
	if(ori_map == NULL)generateOri();
	
	clock_t start_time, finish_time;
	start_time = clock();

	labelSet = new LabelSet(image,ori_map);
	
	int **coverage =  make_2D_int_array(image->m_height,image->m_width);
	//locations of the sites in the partial layers
	float **locations = make_2D_float_array(image->m_height,image->m_width);
	//a complete map of the tiles
	float **wholeMap = make_2D_float_array(image->m_height,image->m_width);
	//all the centers
	int **centerMap = make_2D_int_array(image->m_height,image->m_width);
	int **processed = make_2D_int_array(image->m_height,image->m_width);


	for(int i = 0; i < image->m_height; i++)
		for(int j = 0; j < image->m_width; j++)
		{
			coverage[i][j] = 0;
			centerMap[i][j] = 0;
		}
	bool finish = false;

	int i_width = (int)(sqrt((float)(width*width+height*height))+width+0.5)+1;
	int i_height = (int)(sqrt((float)(width*width+height*height))+height+0.5)+1;
	Node *head = new Node;
	CPoint temp;
	temp.x = 0;
	temp.y = 0;
	head->point = temp;
	head->weight = 0;
	head->next = NULL;
	Node *current = head;
	for(int i = 1; i < image->m_height+image->m_width-1; i++)
	{
		int row,col;
		if(i < image->m_width)
		{
			row = 0; 
			col = i;
		}
		else
		{
			int x = i-image->m_width+1;
			row = x;
			col = image->m_width-1;
		}
		temp.x = col;
		temp.y = row;
		for(;temp.x > 0 && temp.y < image->m_height; temp.x--, temp.y++)
		{
			Node *tempNode = new Node;
			tempNode->point = temp;
			tempNode->weight = 0;
			tempNode->next = NULL;
			current->next = tempNode;
			current = tempNode;
		}
	}

	while(labelSet->getLabelNum() < MAX_LABEL && !finish)
	{
		clock_t start_time, finish_time;
		start_time = clock();

		//generate the whole map, add a condition: coverage < 5
		for(int i = 0; i < image->m_height; i++)
			for(int j = 0; j < image->m_width; j++)
			{
				wholeMap[i][j] = NO_TILE;
				locations[i][j] = OTHER;
				processed[i][j] = UNPROCESSED;
			}


		//To do
		CPoint start;
		start.x = 0;
		start.y = 0;

		CPoint center;
		
		int tNum = 0;
		current = head;
		while(current != NULL)
		{
			center = current->point;
			int i = center.y;
			int j = center.x;
			if(processed[i][j] == UNPROCESSED && centerMap[i][j] == 0)
			{
				if(if_complete(center,ori_map[i][j],image->m_width,image->m_height,width,height))
				{
					//Find a tile and add
					tNum++;
					float ori = ori_map[center.y][center.x];
					wholeMap[center.y][center.x] =  LAY_TILE;
					centerMap[center.y][center.x] = LAY_TILE;
					processed[center.y][center.y] = PROCESSED;
					CPoint ivertex[4];
					CPoint icenter = coordinate_change(center,0,0,ori);
					ivertex[0].x = icenter.x-(int)(i_width/2);
					ivertex[0].y = icenter.y -(int)(i_height/2);
					ivertex[1].x = icenter.x+(int)(i_width/2);
					ivertex[1].y = icenter.y -(int)(i_height/2);
					ivertex[2].x = icenter.x-(int)(i_width/2);
					ivertex[2].y = icenter.y +(int)(i_height/2);
					ivertex[3].x = icenter.x+(int)(i_width/2);
					ivertex[3].y = icenter.y +(int)(i_height/2);

					int minx = 1000000;
					int maxx = -1000000;
					int miny = 1000000;
					int maxy = -1000000;
					CPoint temp;
					for(int k = 0; k < 4; k++)
					{
						temp = inverse_coordinate_change(ivertex[k],0,0,ori);
						if(temp.x < minx)minx = temp.x;
						if(temp.x > maxx)maxx = temp.x;
						if(temp.y < miny)miny = temp.y;
						if(temp.y > maxy)maxy = temp.y;
					}
				
					for(temp.y = miny; temp.y <= maxy; temp.y++)
						for(temp.x = minx; temp.x <= maxx; temp.x++)
						{
							CPoint temp2 = temp;
							if(check_boundary(&temp2,image->m_width,image->m_height))
							{
								if(processed[temp.y][temp.x] == UNPROCESSED && inside(temp,ivertex,0,0,ori) != OUTSIDE)
								{
									if(ifIntersect(center,temp) == true)
									{
										processed[temp.y][temp.x] = PROCESSED;
									}						
								}
							}
						}

				}
				else
				{
					processed[i][j] = PROCESSED;
					centerMap[i][j] = 1;
				}
			}
			current = current->next;
		}

		labelSet->addLabel(width,height,wholeMap);
		
		finish_time = clock();
		double duration = (double)(finish_time-start_time)/CLOCKS_PER_SEC;
		printf("Time:\t %lf seconds, tile Number: %d \n",duration, tNum);
		//visualizeLabel(labelSet->getLabelNum()-1);
	}
	//free_2D_int_array(coverage);
	int tNum = 0;
	for(int i = 0; i < image->m_height; i++)
		for(int j = 0; j < image->m_width; j++)
		{
			if(wholeMap[i][j] == LAY_TILE)
			{
				coverage[i][j] = LAY_TILE;
				tNum++;
			}
			else coverage[i][j] = NO_TILE;
		}
	//printf("tile Number:\t%d\n",tNum);
	free_2D_int_array(centerMap);
	free_2D_float_array(wholeMap);
	free_2D_float_array(locations);
	free_2D_int_array(processed);
	printf("Number of Labels:\t%d\n",labelSet->getLabelNum());
	finish_time = clock();
	double duration = (double)(finish_time-start_time)/CLOCKS_PER_SEC;
	printf("Time of generating layers:\t %lf seconds \n",duration);
	while(head != NULL)
	{
		current = head;
		head = head->next;
		delete current;
	}

	return coverage;
}

int ** Mosaic::buildLabels()
{
	if(image == NULL)
	{
		return NULL;
	}
	if(ori_map == NULL)generateOri();
	
	clock_t start_time, finish_time;
	start_time = clock();

	labelSet = new LabelSet(image,ori_map);
	
	int **coverage =  make_2D_int_array(image->m_height,image->m_width);
	//locations of the sites in the partial layers
	float **locations = make_2D_float_array(image->m_height,image->m_width);
	//a complete map of the tiles
	float **wholeMap = make_2D_float_array(image->m_height,image->m_width);
	//all the centers
	int **centerMap = make_2D_int_array(image->m_height,image->m_width);
	int **processed = make_2D_int_array(image->m_height,image->m_width);


	for(int i = 0; i < image->m_height; i++)
		for(int j = 0; j < image->m_width; j++)
		{
			coverage[i][j] = 0;
			//locations[i][j] = 0;
			centerMap[i][j] = 0;
		}
	bool finish = false;

	int i_width = (int)(sqrt((float)(width*width+height*height))+width+0.5)+1;
	int i_height = (int)(sqrt((float)(width*width+height*height))+height+0.5)+1;
	
	srand(10000);

	while(labelSet->getLabelNum() < MAX_LABEL && !finish)
	{
		clock_t start_time, finish_time;
		start_time = clock();

		//generate the whole map, add a condition: coverage < 5
		for(int i = 0; i < image->m_height; i++)
			for(int j = 0; j < image->m_width; j++)
			{
				wholeMap[i][j] = NO_TILE;
				locations[i][j] = OTHER;
				processed[i][j] = UNPROCESSED;
			}


		//To do
		CPoint start;
		start.x = abs(rand())%image->m_width;
		start.y = abs(rand())%image->m_height;
		while(!if_complete(start,ori_map[start.y][start.x],image->m_width,image->m_height,width,height))
		{
			start.x = abs(rand())%image->m_width;
			start.y = abs(rand())%image->m_height;
		}
		printf("Start Point:\t%d\t%d\n",start.y,start.x);
		start.x = 0;
		start.y = 0;

		CPoint center = start;
		bool find = true;
		
		//up right
		int tNum = 0;
		while(find)
		{
			find = false;
			for(int i = start.y; i >= 0; i--)
			{
				for(int j = start.x; j < image->m_width; j++)
				{
					center.x = j;
					center.y = i;
					if(processed[i][j] == UNPROCESSED && centerMap[i][j] == 0)
					{
						if(if_complete(center,ori_map[i][j],image->m_width,image->m_height,width,height))
						{
							find = true;
							break;
						}
						else
						{
							processed[i][j] = PROCESSED;
							centerMap[i][j] = 1;
						}
					}
				}
				if(find)break;
			}
			if(!find)break;
			tNum++;
			float ori = ori_map[center.y][center.x];
			wholeMap[center.y][center.x] =  LAY_TILE;
			centerMap[center.y][center.x] = LAY_TILE;
			processed[center.y][center.y] = PROCESSED;
			CPoint ivertex[4];
			CPoint icenter = coordinate_change(center,0,0,ori);
			ivertex[0].x = icenter.x-(int)(i_width/2);
			ivertex[0].y = icenter.y -(int)(i_height/2);
			ivertex[1].x = icenter.x+(int)(i_width/2);
			ivertex[1].y = icenter.y -(int)(i_height/2);
			ivertex[2].x = icenter.x-(int)(i_width/2);
			ivertex[2].y = icenter.y +(int)(i_height/2);
			ivertex[3].x = icenter.x+(int)(i_width/2);
			ivertex[3].y = icenter.y +(int)(i_height/2);

			int minx = 1000000;
			int maxx = -1000000;
			int miny = 1000000;
			int maxy = -1000000;
			CPoint temp;
			for(int k = 0; k < 4; k++)
			{
				temp = inverse_coordinate_change(ivertex[k],0,0,ori);
				if(temp.x < minx)minx = temp.x;
				if(temp.x > maxx)maxx = temp.x;
				if(temp.y < miny)miny = temp.y;
				if(temp.y > maxy)maxy = temp.y;
			}
		
			for(temp.y = miny; temp.y <= maxy; temp.y++)
				for(temp.x = minx; temp.x <= maxx; temp.x++)
				{
					CPoint temp2 = temp;
					if(check_boundary(&temp2,image->m_width,image->m_height))
					{
						if(processed[temp.y][temp.x] == UNPROCESSED && inside(temp,ivertex,0,0,ori) != OUTSIDE)
						{
							if(ifIntersect(center,temp) == true)
							{
								processed[temp.y][temp.x] = PROCESSED;
							}						
						}
					}
				}
		}
		//up left
		find = true;
		center = start;
		while(find)
		{
			find = false;
			for(int i = start.y; i >= 0; i--)
			{
				for(int j = start.x; j >= 0; j--)
				{
					center.x = j;
					center.y = i;
					if(processed[i][j] == UNPROCESSED && centerMap[i][j] == 0)
					{
						if(if_complete(center,ori_map[i][j],image->m_width,image->m_height,width,height))
						{
							find = true;
							break;
						}
						else
						{
							processed[i][j] = PROCESSED;
							centerMap[i][j] = 1;
						}
					}
				}
				if(find)break;
			}
			if(!find)break;
			tNum++;
			float ori = ori_map[center.y][center.x];
			wholeMap[center.y][center.x] =  LAY_TILE;
			centerMap[center.y][center.x] = LAY_TILE;
						processed[center.y][center.y] = PROCESSED;
			CPoint ivertex[4];
			CPoint icenter = coordinate_change(center,0,0,ori);
			ivertex[0].x = icenter.x-(int)(i_width/2);
			ivertex[0].y = icenter.y -(int)(i_height/2);
			ivertex[1].x = icenter.x+(int)(i_width/2);
			ivertex[1].y = icenter.y -(int)(i_height/2);
			ivertex[2].x = icenter.x-(int)(i_width/2);
			ivertex[2].y = icenter.y +(int)(i_height/2);
			ivertex[3].x = icenter.x+(int)(i_width/2);
			ivertex[3].y = icenter.y +(int)(i_height/2);

			int minx = 1000000;
			int maxx = -1000000;
			int miny = 1000000;
			int maxy = -1000000;
			CPoint temp;
			for(int k = 0; k < 4; k++)
			{
				temp = inverse_coordinate_change(ivertex[k],0,0,ori);
				if(temp.x < minx)minx = temp.x;
				if(temp.x > maxx)maxx = temp.x;
				if(temp.y < miny)miny = temp.y;
				if(temp.y > maxy)maxy = temp.y;
			}
		
			for(temp.y = miny; temp.y <= maxy; temp.y++)
				for(temp.x = minx; temp.x <= maxx; temp.x++)
				{
					CPoint temp2 = temp;
					if(check_boundary(&temp2,image->m_width,image->m_height))
					{
						if(processed[temp.y][temp.x] == UNPROCESSED && inside(temp,ivertex,0,0,ori) != OUTSIDE)
						{
							if(ifIntersect(center,temp) == true)
							{
								processed[temp.y][temp.x] = PROCESSED;
							}						
						}
					}
				}
		}

		//bottom right
		find = true;
		center = start;
		while(find)
		{
			find = false;
			for(int i = start.y; i < image->m_height; i++)
			{
				for(int j = start.x; j < image->m_width; j++)
				{
					center.x = j;
					center.y = i;
					if(processed[i][j] == UNPROCESSED && centerMap[i][j] == 0)
					{
						if(if_complete(center,ori_map[i][j],image->m_width,image->m_height,width,height))
						{
							find = true;
							break;
						}
						else
						{
							processed[i][j] = PROCESSED;
							centerMap[i][j] = 1;
						}
					}
				}
				if(find)break;
			}
			if(!find)break;
			tNum++;
			float ori = ori_map[center.y][center.x];
			wholeMap[center.y][center.x] =  LAY_TILE;
			centerMap[center.y][center.x] = LAY_TILE;
						processed[center.y][center.y] = PROCESSED;
			CPoint ivertex[4];
			CPoint icenter = coordinate_change(center,0,0,ori);
			ivertex[0].x = icenter.x-(int)(i_width/2);
			ivertex[0].y = icenter.y -(int)(i_height/2);
			ivertex[1].x = icenter.x+(int)(i_width/2);
			ivertex[1].y = icenter.y -(int)(i_height/2);
			ivertex[2].x = icenter.x-(int)(i_width/2);
			ivertex[2].y = icenter.y +(int)(i_height/2);
			ivertex[3].x = icenter.x+(int)(i_width/2);
			ivertex[3].y = icenter.y +(int)(i_height/2);

			int minx = 1000000;
			int maxx = -1000000;
			int miny = 1000000;
			int maxy = -1000000;
			CPoint temp;
			for(int k = 0; k < 4; k++)
			{
				temp = inverse_coordinate_change(ivertex[k],0,0,ori);
				if(temp.x < minx)minx = temp.x;
				if(temp.x > maxx)maxx = temp.x;
				if(temp.y < miny)miny = temp.y;
				if(temp.y > maxy)maxy = temp.y;
			}
		
			for(temp.y = miny; temp.y <= maxy; temp.y++)
				for(temp.x = minx; temp.x <= maxx; temp.x++)
				{
					CPoint temp2 = temp;
					if(check_boundary(&temp2,image->m_width,image->m_height))
					{
						if(processed[temp.y][temp.x] == UNPROCESSED && inside(temp,ivertex,0,0,ori) != OUTSIDE)
						{
							if(ifIntersect(center,temp) == true)
							{
								processed[temp.y][temp.x] = PROCESSED;
							}						
						}
					}
				}
		}

		//bottom left
		find = true;
		center = start;
		while(find)
		{
			find = false;
			for(int i = start.y; i <image->m_height; i++)
			{
				for(int j = start.x; j >= 0; j--)
				{
					center.x = j;
					center.y = i;
					if(processed[i][j] == UNPROCESSED && centerMap[i][j] == 0)
					{
						if(if_complete(center,ori_map[i][j],image->m_width,image->m_height,width,height))
						{
							find = true;
							break;
						}
						else
						{
							processed[i][j] = PROCESSED;
							centerMap[i][j] = 1;
						}
					}
				}
				if(find)break;
			}
			if(!find)break;
			tNum++;
			float ori = ori_map[center.y][center.x];
			wholeMap[center.y][center.x] =  LAY_TILE;
			centerMap[center.y][center.x] = LAY_TILE;
						processed[center.y][center.y] = PROCESSED;
			CPoint ivertex[4];
			CPoint icenter = coordinate_change(center,0,0,ori);
			ivertex[0].x = icenter.x-(int)(i_width/2);
			ivertex[0].y = icenter.y -(int)(i_height/2);
			ivertex[1].x = icenter.x+(int)(i_width/2);
			ivertex[1].y = icenter.y -(int)(i_height/2);
			ivertex[2].x = icenter.x-(int)(i_width/2);
			ivertex[2].y = icenter.y +(int)(i_height/2);
			ivertex[3].x = icenter.x+(int)(i_width/2);
			ivertex[3].y = icenter.y +(int)(i_height/2);

			int minx = 1000000;
			int maxx = -1000000;
			int miny = 1000000;
			int maxy = -1000000;
			CPoint temp;
			for(int k = 0; k < 4; k++)
			{
				temp = inverse_coordinate_change(ivertex[k],0,0,ori);
				if(temp.x < minx)minx = temp.x;
				if(temp.x > maxx)maxx = temp.x;
				if(temp.y < miny)miny = temp.y;
				if(temp.y > maxy)maxy = temp.y;
			}
		
			for(temp.y = miny; temp.y <= maxy; temp.y++)
				for(temp.x = minx; temp.x <= maxx; temp.x++)
				{
					CPoint temp2 = temp;
					if(check_boundary(&temp2,image->m_width,image->m_height))
					{
						if(processed[temp.y][temp.x] == UNPROCESSED && inside(temp,ivertex,0,0,ori) != OUTSIDE)
						{
							if(ifIntersect(center,temp) == true)
							{
								processed[temp.y][temp.x] = PROCESSED;
							}						
						}
					}
				}
		}

		labelSet->addLabel(width,height,wholeMap);
		
		finish_time = clock();
		double duration = (double)(finish_time-start_time)/CLOCKS_PER_SEC;
		printf("Time:\t %lf seconds \n",duration);
	}
	//free_2D_int_array(coverage);
	int tNum = 0;
	for(int i = 0; i < image->m_height; i++)
		for(int j = 0; j < image->m_width; j++)
		{
			if(wholeMap[i][j] == LAY_TILE)
			{
				coverage[i][j] = LAY_TILE;
				tNum++;
			}
			else coverage[i][j] = NO_TILE;
		}
	printf("tile Number:\t%d\n",tNum);
	free_2D_int_array(centerMap);
	free_2D_float_array(wholeMap);
	free_2D_float_array(locations);
	free_2D_int_array(processed);
	printf("Number of Labels:\t%d\n",labelSet->getLabelNum());
	finish_time = clock();
	double duration = (double)(finish_time-start_time)/CLOCKS_PER_SEC;
	printf("Time of generating layers:\t %lf seconds \n",duration);

	//visualizeLabel(0);
	return coverage;
}

void Mosaic::findGap(int FirstLabel, int SecondLabel)
{
	//validate the parameters
	if(labelSet == NULL)buildLabels();
	
	int labelNumber = labelSet->getLabelNum();
	if(FirstLabel < 0 || FirstLabel >= labelNumber)
	{
		if(SecondLabel == 0)FirstLabel = 1;
		else FirstLabel = 0;
	}

	if(SecondLabel < 0 || SecondLabel >= labelNumber)
	{
		if(FirstLabel == 1)SecondLabel = 0;
		else SecondLabel = 1;
	}

	Label *firstLabel = labelSet->getLabel(FirstLabel);
	if(firstLabel == NULL)
	{
		printf("Error in get the whole tile map!\n");
		return;
	}

	Label *secondLabel = labelSet->getLabel(SecondLabel);
	if(secondLabel == NULL)
	{
		printf("Error in get the whole tile map!\n");
		return;
	}

	int **firstMap;
	int **secondMap;

	firstMap = make_2D_int_array(image->m_height,image->m_width);
	secondMap = make_2D_int_array(image->m_height,image->m_width);

	for(int i = 0; i < image->m_height; i++)
		for(int j = 0; j < image->m_width; j++)
		{
			firstMap[i][j]= GAP_REGION;
			secondMap[i][j] = GAP_REGION;
		}


	MNode *tiles;
	int tNumber;

	tiles = firstLabel->returnTiles();
	tNumber = firstLabel->getTileNumber();
	if(tiles == NULL || tNumber < 1)
	{
		printf("Error in get the tile map!\n");
		return;
	}

	for(int i = 0; i < tNumber; i++)
	{
		CPoint center = tiles[i].site;
		float ori = ori_map[center.y][center.x];
		CPoint icenter = coordinate_change(center,0,0,ori);
				//vertex of the tile
		CPoint vertex[4];
		CPoint ivertex[4];
		vertex[0].x = icenter.x-(int)(width/2);
		vertex[0].y = icenter.y-(int)(height/2);
		vertex[1].x = icenter.x+(int)(width/2);
		vertex[1].y = icenter.y-(int)(height/2);
		vertex[2].x = icenter.x-(int)(width/2);
		vertex[2].y = icenter.y+(int)(height/2);
		vertex[3].x = icenter.x+(int)(width/2);
		vertex[3].y = icenter.y+(int)(height/2);

		int minx,miny,maxx,maxy;
		minx = miny = 1000000;
		maxx = maxy = -1;
		for(int k = 0; k < 4; k++)
		{
			ivertex[k] = inverse_coordinate_change(vertex[k],0,0,ori);
			if(ivertex[k].x < minx)minx = ivertex[k].x;
			if(ivertex[k].x > maxx)maxx = ivertex[k].x;
			if(ivertex[k].y < miny)miny = ivertex[k].y;
			if(ivertex[k].y > maxy)maxy = ivertex[k].y;
		}
					
		CPoint temp,itemp;
		for(temp.x = minx; temp.x <= maxx; temp.x++)
			for(temp.y = miny;temp.y <= maxy;temp.y++)
			{
				itemp = temp;
				if(check_boundary(&itemp,image->m_width,image->m_height))
				{
					if(inside(temp,vertex,0,0,ori)==INSIDE)
					{
						firstMap[temp.y][temp.x] = i;
					}
				}
			}
	}

	tiles = secondLabel->returnTiles();
	tNumber = secondLabel->getTileNumber();
	if(tiles == NULL || tNumber < 1)
	{
		printf("Error in get the tile map!\n");
		return;
	}

	for(int i = 0; i < tNumber; i++)
	{
		CPoint center = tiles[i].site;
		float ori = ori_map[center.y][center.x];
		CPoint icenter = coordinate_change(center,0,0,ori);
				//vertex of the tile
		CPoint vertex[4];
		CPoint ivertex[4];
		vertex[0].x = icenter.x-(int)(width/2);
		vertex[0].y = icenter.y-(int)(height/2);
		vertex[1].x = icenter.x+(int)(width/2);
		vertex[1].y = icenter.y-(int)(height/2);
		vertex[2].x = icenter.x-(int)(width/2);
		vertex[2].y = icenter.y+(int)(height/2);
		vertex[3].x = icenter.x+(int)(width/2);
		vertex[3].y = icenter.y+(int)(height/2);

		int minx,miny,maxx,maxy;
		minx = miny = 1000000;
		maxx = maxy = -1;
		for(int k = 0; k < 4; k++)
		{
			ivertex[k] = inverse_coordinate_change(vertex[k],0,0,ori);
			if(ivertex[k].x < minx)minx = ivertex[k].x;
			if(ivertex[k].x > maxx)maxx = ivertex[k].x;
			if(ivertex[k].y < miny)miny = ivertex[k].y;
			if(ivertex[k].y > maxy)maxy = ivertex[k].y;
		}
					
		CPoint temp,itemp;
		for(temp.x = minx; temp.x <= maxx; temp.x++)
			for(temp.y = miny;temp.y <= maxy;temp.y++)
			{
				itemp = temp;
				if(check_boundary(&itemp,image->m_width,image->m_height))
				{
					if(inside(temp,vertex,0,0,ori)==INSIDE)
					{
						secondMap[temp.y][temp.x] = i;
					}
				}
			}
	}

	int commonGap = 0;
	int firstGap = 0;
	int secondGap = 0;
	
	int col = image->m_width;
	int row = image->m_height;
	MyImage *im = new MyImage(row,col,GRAY,BMP_TYPE,BYTE_FORMAT);
	
	for(int i = 0; i < image->m_height; i++)
		for(int j = 0; j < image->m_width; j++)
		{
			if(firstMap[i][j] == GAP_REGION && secondMap[i][j] == GAP_REGION)
			{
				im->m_data[i][j] = 0;
				commonGap++;
			}
			else 
			{
				im->m_data[i][j] = 254;
				if(firstMap[i][j] == GAP_REGION)firstGap++;
				else if(secondMap[i][j] == GAP_REGION)secondGap++;
			}
		}

	float firstRatiol = (float)commonGap/(float)(firstGap);
	float secondRatiol = (float)commonGap/(float)(secondGap);
	printf("Common gap in first map:\t%f\nCommon gap in second map:\t%f\n",firstRatiol, secondRatiol);

	free_2D_int_array(firstMap);
	free_2D_int_array(secondMap);

	im->WriteImage("commonGap.bmp");

}