#include "stdafx.h"
#include "gloabalFunctions.h"
#include "stdlib.h"

float **make_2D_float_array(int row,int col)
{
	float *arr_ptr;
	float **arr;
	int i;
	if ((arr=(float **)malloc(row*sizeof(float *)))==NULL)
    {AfxMessageBox("Can't allocate core in make_2D_float_array",MB_OK); return(NULL);}
	if ((arr_ptr=(float *)malloc(row*col*sizeof(float)))==NULL)
    {AfxMessageBox("Can't allocate core in make_2D_float_array",MB_OK); return(NULL);}
	for(i=0;i<row;i++) arr[i]=&arr_ptr[i*col];
	return(arr);
} 

void free_2D_float_array(float **arr)
{
	free((char*)arr[0]);
	free((char*)arr);
}

int **make_2D_int_array(int row,int col)
{
	int *arr_ptr;
	int **arr;
	int i;
	if ((arr=(int **)malloc(row*sizeof(int *)))==NULL)
    {AfxMessageBox("Can't allocate core in make_2D_int_array",MB_OK); return(NULL);}
	if ((arr_ptr=(int *)malloc(row*col*sizeof(int)))==NULL)
    {AfxMessageBox("Can't allocate core in make_2D_int_array",MB_OK); return(NULL);}
	for(i=0;i<row;i++) arr[i]=&arr_ptr[i*col];
	return(arr);
}

void free_2D_int_array(int **arr)
{
	free((char*)arr[0]);
	free((char*)arr);
}

CPoint coordinate_change(CPoint p, int shift_h, int shift_v, float ori)
{
	CPoint ip;
	//ip.x = (int)((p.x-shift_h)*cosf(ori)+(p.y-shift_v)*sinf(ori)+0.5);
	//ip.y = (int)((p.y-shift_v)*cosf(ori)-(p.x-shift_h)*sinf(ori)+0.5);
	ip.x = (int)(p.x*cosf(ori)+p.y*sinf(ori)-shift_h+0.5);
	ip.y = (int)(p.y*cosf(ori)-p.x*sinf(ori)-shift_v+0.5);
	return ip;
}

CPoint inverse_coordinate_change(CPoint ip, int shift_h, int shift_v, float ori)
{
	CPoint p;
	//p.x = (int)(ip.x*cosf(ori)-ip.y*sinf(ori)+shift_v+0.5);
	//p.y = (int)(ip.x*sinf(ori)+ip.y*cosf(ori)+shift_h+0.5);
	p.x = (int)((ip.x+shift_h)*cosf(ori)-(ip.y+shift_v)*sinf(ori)+0.5);
	p.y = (int)((ip.x+shift_h)*sinf(ori)+(ip.y+shift_v)*cosf(ori)+0.5);
	return p;
}

//if inside the image ,return true
bool check_boundary(CPoint *p, int IMAGE_WIDTH, int IMAGE_HEIGHT)
{
	bool inside = true;
	
	if(p->x < 0)
	{
		p->x = 0;
		inside = false;
	}
	if(p-> x > IMAGE_WIDTH-1)
	{
		p->x = IMAGE_WIDTH-1;
		inside = false;
	}

	if(p->y < 0)
	{
		p->y = 0;
		inside = false;
	}
	if(p->y > IMAGE_HEIGHT-1)
	{
		p->y = IMAGE_HEIGHT-1;
		inside = false;
	}
	return inside;

}

int inside(CPoint p, CPoint tile[4], int shift_h, int shift_v, float ori)
{
	float location[4][2];
	location[0][0] = (p.x-0.5)*cosf(ori)+(p.y-0.5)*sinf(ori)-shift_h;
	location[0][1] = (p.y-0.5)*cosf(ori)-(p.x-0.5)*sinf(ori)-shift_v;
	location[1][0] = (p.x+0.5)*cosf(ori)+(p.y-0.5)*sinf(ori)-shift_h;
	location[1][1] = (p.y-0.5)*cosf(ori)-(p.x+0.5)*sinf(ori)-shift_v;
	location[2][0] = (p.x-0.5)*cosf(ori)+(p.y+0.5)*sinf(ori)-shift_h;
	location[2][1] = (p.y+0.5)*cosf(ori)-(p.x-0.5)*sinf(ori)-shift_v;
	location[3][1] = (p.y+0.5)*cosf(ori)-(p.x+0.5)*sinf(ori)-shift_v;
	location[3][0] = (p.x+0.5)*cosf(ori)+(p.y+0.5)*sinf(ori)-shift_h;

	int num = 0;
	for(int i = 0; i < 4; i++)
	{
		if(location[i][0] < tile[0].x || location[i][0] > tile[3].x || location[i][1] < tile[0].y || location[i][1] > tile[3].y)num++; 
	}
	if( num == 0)return INSIDE;
	if( num == 1)return BORDER;
	return OUTSIDE;
}

float **compute_gradient(MyImage *im, float ***edge)
{
	if(im == NULL)return NULL;
	int width = im->m_width;
	int height = im->m_height;
	float **gradient = make_2D_float_array(height,width);
	*edge = make_2D_float_array(height,width);
	for(int i = 0; i < height; i++)
		for(int j = 0; j < width; j++)
		{
			gradient[i][j] = 0;
			(*edge)[i][j] = 0;
		}

	float u[3],v[3];
	CPoint p[3][3];
	float gxx, gxy, gyy,angle,div;
	for(int i = 1; i < height-1; i++)
		for(int j = 1; j < width-1; j++)
		{	
			gxx = gyy = gxy = 0;
			for(int k = 0; k < 3;k++)
				for(int l = 0; l < 3; l++)
				{
					p[k][l].x = j+l-1;
					p[k][l].y = i+k-1;
				}
			u[0] = ((imRef_R(im,p[0][0])-imRef_R(im,p[0][2]))+2*(imRef_R(im,p[1][0])-imRef_R(im,p[1][2]))+(imRef_R(im,p[2][0])-imRef_R(im,p[2][2])))/4;
			u[1] = ((imRef_G(im,p[0][0])-imRef_G(im,p[0][2]))+2*(imRef_G(im,p[1][0])-imRef_G(im,p[1][2]))+(imRef_G(im,p[2][0])-imRef_G(im,p[2][2])))/4;
			u[2] = ((imRef_B(im,p[0][0])-imRef_B(im,p[0][2]))+2*(imRef_B(im,p[1][0])-imRef_B(im,p[1][2]))+(imRef_B(im,p[2][0])-imRef_B(im,p[2][2])))/4;

			v[0] = ((imRef_R(im,p[0][0])-imRef_R(im,p[2][0]))+2*(imRef_R(im,p[0][1])-imRef_R(im,p[2][1]))+(imRef_R(im,p[0][2])-imRef_R(im,p[2][2])))/4;
			v[1] = ((imRef_G(im,p[0][0])-imRef_G(im,p[2][0]))+2*(imRef_G(im,p[0][1])-imRef_G(im,p[2][1]))+(imRef_G(im,p[0][2])-imRef_G(im,p[2][2])))/4;
			v[2] = ((imRef_B(im,p[0][0])-imRef_B(im,p[2][0]))+2*(imRef_B(im,p[0][1])-imRef_B(im,p[2][1]))+(imRef_B(im,p[0][2])-imRef_B(im,p[2][2])))/4;

			for( int l = 0; l < 3; l++)
			{
				gxx += u[l]*u[l];
				gyy += v[l]*v[l];
				gxy += u[l]*v[l];
			}
			angle = 0.5*atan2(2*gxy,(gxx-gyy));
			//if(angle > 0)(*edge)[i][j] = angle;
			//else (*edge)[i][j] = angle+PI;
			(*edge)[i][j] = angle;
			gradient[i][j] = sqrt(0.5*((gxx+gyy)+(gxx-gyy)*cos(2*angle)+2*gxy*sin(2*angle)));
		}
	return gradient;
}

//return true if the tile is complete
bool if_complete(CPoint center, float ori, int image_width, int image_height, int width, int height)
{
	CPoint icenter = coordinate_change(center,0,0,ori);
	CPoint vertex[4];
	vertex[0].x = icenter.x-(int)(width/2);
	vertex[0].y = icenter.y-(int)(height/2);
	vertex[1].x = icenter.x+(int)(width/2);
	vertex[1].y = icenter.y-(int)(height/2);
	vertex[2].x = icenter.x-(int)(width/2);
	vertex[2].y = icenter.y+(int)(height/2);
	vertex[3].x = icenter.x+(int)(width/2);
	vertex[3].y = icenter.y+(int)(height/2);

	for(int i = 0; i < 4; i++)
	{
		vertex[i] = inverse_coordinate_change(vertex[i],0,0,ori);
		if(!check_boundary(&(vertex[i]),image_width,image_height))return false;
	}
	return true;
}

float **image_rotate(float **image,int width, int height,float angle, int *nwidth, int *nheight)
{
	if(image == NULL || width < 10 || height < 10 || nwidth == NULL || nheight == NULL)
	{
		return NULL;
	}

	CPoint vertex[4];
	vertex[0].x = 0;
	vertex[0].y = 0;
	vertex[1].x = width-1;
	vertex[1].y = 0;
	vertex[2].x = 0;
	vertex[2].y = height-1;
	vertex[3].x = width-1;
	vertex[3].y = height-1;

	CPoint ivertex[4];
	int maxX, maxY;
	int minX, minY;
	maxX = maxY = -1000000;
	minX = minY = 1000000;

	for( int i = 0; i < 4; i++)
	{
		ivertex[i] = coordinate_change(vertex[i],0,0,angle);
		if(maxX < ivertex[i].x )maxX = ivertex[i].x;
		if(maxY < ivertex[i].y )maxY = ivertex[i].y;
		if(minX > ivertex[i].x )minX = ivertex[i].x;
		if(minY > ivertex[i].y )minY = ivertex[i].y;
	}

	int shift_v, shift_h;
	shift_v = minY;
	shift_h = minX;

	*nwidth = abs(maxX-minX);
	*nheight = abs(maxY-minY);

	float **rimage = make_2D_float_array(*nheight,*nwidth);

	for(int i = 0; i < *nheight; i++)
		for(int j = 0; j < *nwidth; j++)
		{
			rimage[i][j] = 0;
			CPoint temp,temp2;
			temp.x = j;
			temp.y = i;
			temp2 = inverse_coordinate_change(temp,shift_h,shift_v,angle);
			if(check_boundary(&temp2,width,height))
			{
				rimage[i][j] = image[temp2.y][temp2.x];
			}
		}

	return rimage;
}

float **computeDistanceMap(float **func, int width, int height)
{
	if(func == NULL || width < 10 || height < 10)return NULL;
	
	float **distance_map = make_2D_float_array(height,width);

	for( int i = 0; i < height; i++)
		for( int j = 0; j < width; j++)
		{
			distance_map[i][j] = func[i][j];
		}

		//distance transform
		float dis[2][2];
		float min;
		//forward
		for( int i = 0; i < height; i++)
			for( int j = 0; j < width; j++)
			{
				CPoint temp;
				temp.x = j-1;
				temp.y = i-1;
				if(check_boundary(&temp,width,height))dis[0][0] = distance_map[i-1][j-1]+1.4;
				else dis[0][0] = 1000000;
				
				temp.x = j;
				temp.y = i-1;
				if(check_boundary(&temp,width,height))dis[0][1] = distance_map[i-1][j]+1;
				else dis[0][1] = 1000000;

				temp.x = j-1;
				temp.y = i;
				if(check_boundary(&temp,width,height))dis[1][0] = distance_map[i][j-1]+1;
				else dis[1][0] = 1000000;

				dis[1][1] = distance_map[i][j];
				min = 100000;
//				printf("min:\t%d\n",min);
				for( int k = 0; k < 2; k++)
					for(int l = 0; l < 2; l++)
					{
						if(dis[k][l] < min) min = dis[k][l];
					}
//				printf("min:\t%d\n",min);
				distance_map[i][j] = min;
			}
		//backward
		for( int i = height-1; i >= 0; i--)
			for( int j = width-1; j >= 0 ; j--)
			{
				CPoint temp;
			
				dis[0][0] = distance_map[i][j];

				temp.x = j+1;
				temp.y = i;
				if(check_boundary(&temp,width,height))dis[0][1] = distance_map[i][j+1]+1;
				else dis[0][1] = 1000000;

				temp.x = j;
				temp.y = i+1;
				if(check_boundary(&temp,width,height))dis[1][0] = distance_map[i+1][j]+1;
				else dis[1][0] = 1000000;

				temp.x = j+1;
				temp.y = i+1;
				if(check_boundary(&temp,width,height))dis[1][1] = distance_map[i+1][j+1]+1.4;
				else dis[1][1] = 1000000;

				min = 100000;
				for( int k = 0; k < 2; k++)
					for(int l = 0; l < 2; l++)
					{
						if(dis[k][l] < min) min = dis[k][l];
					}
				distance_map[i][j] = min;
			}

/*	float **distance_map2 = make_2D_float_array(height,width);

	int *v = (int *)malloc(sizeof(int)*width);
	float *z = (float *)malloc(sizeof(float)*width);

	//DT of first dimension
	for(int row = 0; row < height; row++)
	{
		int k = 0;
		v[0] = 0;
		z[0] = -1000000;
		z[1] = 1000000;
		for(int q = 1; q < width; q++)
		{
			float s = (float)((func[row][q]+q*q)-(func[row][v[k]]+v[k]*v[k]))/(float)(2*(q-v[k]));
			while(s <= z[k]&&k>=0)
			{
				k = k-1;
				s = (float)((func[row][q]+q*q)-(func[row][v[k]]+v[k]*v[k]))/(float)(2*(q-v[k]));
			}
			k = k+1;
			v[k] = q;
			z[k] = s;
			z[k+1] = 1000000;
		}

		k = 0;
		for( int q = 0; q < width; q++)
		{
			while(z[k+1] < q)k = k+1;
			distance_map[row][q] = (q-v[k])*(q-v[k])+func[row][v[k]];
		}
	}

//	free(v);
//	free((void *)z);

	v = (int *)malloc(sizeof(int)*height);
	z = (float *)malloc(sizeof(float)*height);

	//DT of second dimension
	for(int col = 0; col < width; col++)
	{
		int k = 0;
		v[0] = 0;
		z[0] = -1000000;
		z[1] = 1000000;
		for(int q = 1; q < height; q++)
		{
			float s = (float)((distance_map[q][col]+q*q)-(distance_map[v[k]][col]+v[k]*v[k]))/(float)(2*(q-v[k]));
			while(s <= z[k]&&k >= 0)
			{
				k = k-1;
				s = (float)((distance_map[q][col]+q*q)-(distance_map[v[k]][col]+v[k]*v[k]))/(float)(2*(q-v[k]));
			}
			k = k+1;
			v[k] = q;
			z[k] = s;
			z[k+1] = 1000000;
		}

		k = 0;
		for( int q = 0; q < height; q++)
		{
			while(z[k+1] < q)k = k+1;
			distance_map2[q][col] = (q-v[k])*(q-v[k])+distance_map[q][v[k]];
		}
	}

//	free(v);
//	free(z);

	//free_2D_float_array(distance_map);
	return distance_map2;*/
	return distance_map;
}

//return the tileSize/regionSize
float computeGap(float **ori_map, int **process, CPoint center, int image_width, int image_height,int width,int height,int region_size, int tile_number)
{
	if(ori_map == NULL || process == NULL || image_width < 10 || image_height < 10)return -1;
	if(!check_boundary(&center,image_width,image_height))return -1;
	float ori = ori_map[center.y][center.x];

	int i_width = (int)(sqrt((float)(width*width+height*height))+width+0.5)+1;
	int i_height = (int)(sqrt((float)(width*width+height*height))+height+0.5)+1;

		CPoint temp;
		CPoint icenter = coordinate_change(center,0,0,ori);
		CPoint ivertex[4];
		ivertex[0].x = icenter.x-(int)(i_width/2);
		ivertex[0].y = icenter.y -(int)(i_height/2);
		ivertex[1].x = icenter.x+(int)(i_width/2);
		ivertex[1].y = icenter.y -(int)(i_height/2);
		ivertex[2].x = icenter.x-(int)(i_width/2);
		ivertex[2].y = icenter.y +(int)(i_height/2);
		ivertex[3].x = icenter.x+(int)(i_width/2);
		ivertex[3].y = icenter.y +(int)(i_height/2);

		int minx = 1000000;
		int maxx = -1000000;
		int miny = 1000000;
		int maxy = -1000000;
		
		for(int k = 0; k < 4; k++)
		{
			temp = inverse_coordinate_change(ivertex[k],0,0,ori);
			if(temp.x < minx)minx = temp.x;
			if(temp.x > maxx)maxx = temp.x;
			if(temp.y < miny)miny = temp.y;
			if(temp.y > maxy)maxy = temp.y;
		}

		for(temp.y = miny; temp.y <= maxy; temp.y++)
			for(temp.x = minx; temp.x <= maxx; temp.x++)
			{
				CPoint temp2 = temp;
				if(check_boundary(&temp2,image_width,image_height))
				{
					if(process[temp.y][temp.x] == UNPROCESSED && inside(temp,ivertex,0,0,ori) != OUTSIDE)
					{
						if(if_Intersect(ori_map,center,temp,image_width,image_height,width,height) == true)
						{
							region_size++;
						}
					}
				}
			}
		return (float)((tile_number+1)*width*height)/(float)region_size;
}

bool if_Intersect(float **ori_map, CPoint center1, CPoint center2, int image_width, int image_height,int width, int height)
{
	if( ori_map == NULL)
	{
		printf("No orienatation map!\n");
		return true;
	}

	float dis = (center1.x-center2.x)*(center1.x-center2.x)+(center1.y-center2.y)*(center1.y-center2.y);
	if(dis > (width*width+height*height))return false;

	double ori1, ori2;
	if(check_boundary(&center1,image_width,image_height))ori1 = (double)ori_map[center1.y][center1.x];
	else return false;
	if(check_boundary(&center2,image_width,image_height))ori2 = (double)ori_map[center2.y][center2.x];
	else return false;

//		ip.x = (int)(p.x*cosf(ori)+p.y*sinf(ori)-shift_h+0.5);
//		ip.y = (int)(p.y*cosf(ori)-p.x*sinf(ori)-shift_v+0.5);
	double icenter1x,icenter1y;
	icenter1x = (double)center1.x*cosf(ori1)+(double)center1.y*sinf(ori1);
	icenter1y = (double)center1.y*cosf(ori1)-(double)center1.x*sinf(ori1);
	double icenter2x,icenter2y;
	icenter2x = (double)center2.x*cosf(ori2)+(double)center2.y*sinf(ori2);
	icenter2y = (double)center2.y*cosf(ori2)-(double)center2.x*sinf(ori2);

	double icenter21x,icenter21y;
	icenter21x = (double)center2.x*cosf(ori1)+(double)center2.y*sinf(ori1);
	icenter21y = (double)center2.y*cosf(ori1)-(double)center2.x*sinf(ori1);
	dis = sqrt((icenter21x-icenter1x)*(icenter21x-icenter1x)+(icenter21y-icenter1y)*(icenter21y-icenter1y));
	if(dis <(double)width/2 && dis <(double)height/2)return true;
	//if(fabs(icenter21x-icenter1x)<(float)width && fabs(icenter21y - icenter1y) < (float)height)return true;

	double ivertex1x[4];
	double ivertex1y[4];
	ivertex1x[0] = icenter1x-((double)width/2);
	ivertex1y[0] = icenter1y-((double)height/2);
	ivertex1x[1] = icenter1x+((double)width/2)-1;
	ivertex1y[1] = icenter1y-((double)height/2);
	ivertex1x[2] = icenter1x-((double)width/2);
	ivertex1y[2] = icenter1y+((double)height/2)-1;
	ivertex1x[3] = icenter1x+((double)width/2)-1;
	ivertex1y[3] = icenter1y+((double)height/2)-1;

	double ivertex2x[4];
	double ivertex2y[4];
	ivertex2x[0] = icenter2x-((double)width/2);
	ivertex2y[0] = icenter2y-((double)height/2);
	ivertex2x[1] = icenter2x+((double)width/2)-1;
	ivertex2y[1] = icenter2y-((double)height/2);
	ivertex2x[2] = icenter2x-((double)width/2);
	ivertex2y[2] = icenter2y+((double)height/2)-1;
	ivertex2x[3] = icenter2x+((double)width/2)-1;
	ivertex2y[3] = icenter2y+((double)height/2)-1;


	for(int k = 0; k < 4; k++)
	{
		//CPoint temp = inverse_coordinate_change(ivertex1[k],0,0,ori1);
		//temp = coordinate_change(temp,0,0,ori2);
		double tempx,tempy;
		double temp2x,temp2y;
//		p.x = (int)((ip.x+shift_h)*cosf(ori)-(ip.y+shift_v)*sinf(ori)+0.5);
//		p.y = (int)((ip.x+shift_h)*sinf(ori)+(ip.y+shift_v)*cosf(ori)+0.5);
//		ip.x = (int)(p.x*cosf(ori)+p.y*sinf(ori)-shift_h+0.5);
//		ip.y = (int)(p.y*cosf(ori)-p.x*sinf(ori)-shift_v+0.5);

		temp2x = (double)(ivertex1x[k]*cosf(ori1)-ivertex1y[k]*sinf(ori1));
		temp2y = (double)(ivertex1x[k]*sinf(ori1)+ivertex1y[k]*cosf(ori1));
		tempx =(double)(temp2x*cosf(ori2)+temp2y*sinf(ori2));
		tempy =(double)(temp2y*cosf(ori2)-temp2x*sinf(ori2));
		//if(tempx <= ivertex2x[3] && tempx >= ivertex2x[0] && tempy <= ivertex2y[3] && tempy >= ivertex2y[0])return true;
		if(fabs(tempx-icenter2x)-(double)width/2 < -0.5 && fabs(tempy-icenter2y)-(double)height/2 < -0.5)return true;

		temp2x = (double)(ivertex2x[k]*cosf(ori2)-ivertex2y[k]*sinf(ori2));
		temp2y = (double)(ivertex2x[k]*sinf(ori2)+ivertex2y[k]*cosf(ori2));
		tempx =(double)(temp2x*cosf(ori1)+temp2y*sinf(ori1));
		tempy =(double)(temp2y*cosf(ori1)-temp2x*sinf(ori1));
		//if(tempx <= ivertex1x[3] && tempx >= ivertex1x[0] && tempy <= ivertex1y[3] && tempy >= ivertex1y[0])return true;
		if(fabs(tempx-icenter1x)-(double)width/2 < -0.5 && fabs(tempy-icenter1y)-(double)height/2 < -0.5)return true;
	}
	return false;
}