#include "MyImage.h"

#include "stdafx.h"
#include "gloabalFunctions.h"
#include "integral_image.h"

IntegralImage::IntegralImage()
{
	height = 0;
	width = 0;
	iheight = 0;
	iwidth = 0;
	ori = 0;
	shift_h = 0;
	shift_v = 0;
	image = NULL;
	integral_image = NULL;

}

IntegralImage::~IntegralImage()
{

}

void IntegralImage::computeIntegralImage(float **iImage,int oriWidth,int oriHeight,float angle)
{
	if(iImage == NULL || oriWidth < 10 || oriHeight < 10)return;

	height = oriHeight;
	width = oriWidth;
	image = iImage;
	ori = angle;

	CPoint vertex[4];
	vertex[0].x = 0;
	vertex[0].y = 0;
	vertex[1].x = width-1;
	vertex[1].y = 0;
	vertex[2].x = 0;
	vertex[2].y = height-1;
	vertex[3].x = width-1;
	vertex[3].y = height-1;

	CPoint ivertex[4];
	int maxX, maxY;
	int minX, minY;
	maxX = maxY = -1000000;
	minX = minY = 1000000;

	for( int i = 0; i < 4; i++)
	{
		ivertex[i] = coordinate_change(vertex[i],0,0,angle);
		if(maxX < ivertex[i].x )maxX = ivertex[i].x;
		if(maxY < ivertex[i].y )maxY = ivertex[i].y;
		if(minX > ivertex[i].x )minX = ivertex[i].x;
		if(minY > ivertex[i].y )minY = ivertex[i].y;
	}

	shift_v = minY;
	shift_h = minX;

	float **rImage = image_rotate(image,width,height,ori,&iwidth, &iheight);
	integral_image = make_2D_float_array(iheight,iwidth);

	CPoint temp;
	CPoint tempA,tempB,tempC;
	float A,B,C;

	for(temp.y = 0; temp.y < iheight; temp.y++)
		for(temp.x = 0; temp.x < iwidth; temp.x++)
		{
			tempA = tempB = tempC = temp;
			tempA.x--;
			tempB.y--;
			tempC.x--;
			tempC.y--;

			if(check_boundary(&tempA,iwidth,iheight))A = integral_image[tempA.y][tempA.x];
			else A = 0;
			if(check_boundary(&tempB,iwidth,iheight))B = integral_image[tempB.y][tempB.x];
			else B = 0;
			if(check_boundary(&tempC,iwidth,iheight))C = integral_image[tempC.y][tempC.x];
			else C = 0;

			integral_image[temp.y][temp.x] = rImage[temp.y][temp.x]+A+B-C;
		}
	free_2D_float_array(rImage);
}

float IntegralImage::getValue(CPoint icenter,int bwidth,int bheight)
{
	if(integral_image == NULL)return 0;
	icenter.x -= shift_h;
	icenter.y -= shift_v;
	CPoint ivertex[4];
	ivertex[0].x = icenter.x-(int)bwidth/2;
	ivertex[0].y = icenter.y-(int)bheight/2;
	ivertex[1].x = icenter.x+(int)bwidth/2;
	ivertex[1].y = icenter.y-(int)bheight/2;
	ivertex[2].x = icenter.x-(int)bwidth/2;
	ivertex[2].y = icenter.y+(int)bheight/2;
	ivertex[3].x = icenter.x+(int)bwidth/2;
	ivertex[3].y = icenter.y+(int)bheight/2;

	float value[4];
	for(int i = 0; i < 4; i++)
	{
		if(check_boundary(&ivertex[i],iwidth,iheight))value[i] = integral_image[ivertex[i].y][ivertex[i].x];
		else value[i] = 0;
	}

	return (value[3]-value[1]-value[2]+value[0]);
}